@@ -1,6 +1,5 @@
 from __future__ import unicode_literals
 from distutils.version import StrictVersion
-from errno import EWOULDBLOCK
 from itertools import chain
 from time import time
 import io
@@ -10,12 +9,6 @@ import sys
 import threading
 import warnings
 
-try:
-    import ssl
-    ssl_available = True
-except ImportError:
-    ssl_available = False
-
 from redis._compat import (xrange, imap, byte_to_chr, unicode, long,
                            nativestr, basestring, iteritems,
                            LifoQueue, Empty, Full, urlparse, parse_qs,
@@ -34,6 +27,25 @@ from redis.exceptions import (
     TimeoutError,
 )
 from redis.utils import HIREDIS_AVAILABLE
+
+try:
+    import ssl
+    ssl_available = True
+except ImportError:
+    ssl_available = False
+
+if ssl_available:
+    # note that when using nonblocking sockets over ssl, the ssl module
+    # raises its own exceptions rather than the normal BlockingIOError
+    blocking_exceptions = (
+        BlockingIOError,
+        ssl.SSLWantReadError,
+        ssl.SSLWantWriteError
+    )
+else:
+    blocking_exceptions = (BlockingIOError,)
+
+
 if HIREDIS_AVAILABLE:
     import hiredis
 
@@ -168,12 +180,12 @@ class SocketBuffer(object):
                 if length is not None and length > marker:
                     continue
                 return True
-        except BlockingIOError as ex:
+        except blocking_exceptions as ex:
             # if we're in nonblocking mode and the recv raises a
             # blocking error, simply return False indicating that
             # there's no data to be read. otherwise raise the
             # original exception.
-            if raise_on_timeout or ex.errno != EWOULDBLOCK:
+            if raise_on_timeout:
                 raise
             return False
         except socket.timeout:
@@ -397,12 +409,12 @@ class HiredisParser(BaseParser):
             # data was read from the socket and added to the buffer.
             # return True to indicate that data was read.
             return True
-        except BlockingIOError as ex:
+        except blocking_exceptions as ex:
             # if we're in nonblocking mode and the recv raises a
             # blocking error, simply return False indicating that
             # there's no data to be read. otherwise raise the
             # original exception.
-            if raise_on_timeout or ex.errno != EWOULDBLOCK:
+            if raise_on_timeout:
                 raise
             return False
         except socket.timeout:
