@@ -212,6 +212,48 @@ class TestPubSubSubscribeUnsubscribe(object):
             assert message is None
         assert p.subscribed is False
 
+    def test_sub_unsub_resub_channels(self, r):
+        kwargs = make_subscribe_test_data(r.pubsub(), 'channel')
+        self._test_sub_unsub_resub(**kwargs)
+
+    def test_sub_unsub_resub_patterns(self, r):
+        kwargs = make_subscribe_test_data(r.pubsub(), 'pattern')
+        self._test_sub_unsub_resub(**kwargs)
+
+    def _test_sub_unsub_resub(self, p, sub_type, unsub_type, sub_func,
+                              unsub_func, keys):
+        # https://github.com/andymccurdy/redis-py/issues/764
+        key = keys[0]
+        sub_func(key)
+        unsub_func(key)
+        sub_func(key)
+        assert p.subscribed is True
+        assert wait_for_message(p) == make_message(sub_type, key, 1)
+        assert wait_for_message(p) == make_message(unsub_type, key, 0)
+        assert wait_for_message(p) == make_message(sub_type, key, 1)
+        assert p.subscribed is True
+
+    def test_sub_unsub_all_resub_channels(self, r):
+        kwargs = make_subscribe_test_data(r.pubsub(), 'channel')
+        self._test_sub_unsub_all_resub(**kwargs)
+
+    def test_sub_unsub_all_resub_patterns(self, r):
+        kwargs = make_subscribe_test_data(r.pubsub(), 'pattern')
+        self._test_sub_unsub_all_resub(**kwargs)
+
+    def _test_sub_unsub_all_resub(self, p, sub_type, unsub_type, sub_func,
+                                  unsub_func, keys):
+        # https://github.com/andymccurdy/redis-py/issues/764
+        key = keys[0]
+        sub_func(key)
+        unsub_func()
+        sub_func(key)
+        assert p.subscribed is True
+        assert wait_for_message(p) == make_message(sub_type, key, 1)
+        assert wait_for_message(p) == make_message(unsub_type, key, 0)
+        assert wait_for_message(p) == make_message(sub_type, key, 1)
+        assert p.subscribed is True
+
 
 class TestPubSubMessages(object):
     def setup_method(self, method):
@@ -448,3 +490,16 @@ class TestPubSubPings(object):
         assert wait_for_message(p) == make_message(type='pong', channel=None,
                                                    data='hello world',
                                                    pattern=None)
+
+
+class TestPubSubConnectionKilled(object):
+
+    @skip_if_server_version_lt('3.0.0')
+    def test_connection_error_raised_when_connection_dies(self, r):
+        p = r.pubsub(ignore_subscribe_messages=True)
+        p.subscribe('foo')
+        for client in r.client_list():
+            if client['cmd'] == 'subscribe':
+                r.client_kill_filter(_id=client['id'])
+        with pytest.raises(ConnectionError):
+            wait_for_message(p)
